﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.Diagnostics;
using System.IO.Ports;
using System.Linq;
using System.Text;
using System.IO;
using System.Threading;
using Microsoft.Win32;

namespace AmpDelayService
{
    public class AmpWorker
    {
        private Thread _workerThread;
        private bool _isRunning = false;
        private SerialPort _comm = null;
        private const int TimeOutMS = 1500;
        private string _delayString = "";
        private int _delayMS = 5000;
        private int _HBCount = 2;
        private StringBuilder _commandBuffer;
        private DateTime _nextPing;
        private DateTime _lastPing;

        ~AmpWorker()
        {
            Debug.WriteLine("Disposing AmpWorker");
            Cleanup();
        }

        public void Start()
        {
            if (_workerThread != null)
                Stop();
            
            string HBCount = ConfigurationManager.AppSettings["HBCOUNT"];
            int.TryParse(HBCount, out _HBCount);
            if (_HBCount < 2)
                _HBCount = 2;

            _commandBuffer = new StringBuilder();
            _isRunning = true;
            _lastPing = DateTime.Now;
            _nextPing = DateTime.Now.AddMilliseconds(_delayMS / _HBCount);

            _workerThread = new Thread(ListenLoop);
            _workerThread.Start();

            Thread.Sleep(1000);
        }

        public void Stop()
        {
            
            if (_workerThread != null)
            {
                _isRunning = false;

                Thread.Sleep(8000); //wait till ended

                Cleanup();

                try
                {
                    _workerThread.Abort();
                }
                catch
                {
                    
                    
                }
             
                _workerThread = null;    
            }

            

        }

        private void ListenLoop()
        {
           

            do
            {
                try
                {
                    if (_comm == null || !_comm.IsOpen)
                    {
                        InitPortComm();
                    }

                    if (_comm != null && _comm.IsOpen)
                    {


                        //check for buffer
                        if (_comm.BytesToRead > 0)
                        {
                            string buffer = _comm.ReadExisting();
                            _commandBuffer.Append(buffer);
                            if (buffer.IndexOf('\n') >= 0)
                            {
                                ProcessCommands();
                            }
                        }


                        if (DateTime.Now > _nextPing)
                        {
                            Debug.WriteLine(String.Format("{0}:PING", DateTime.Now));

                            _comm.WriteLine("PING");

                            _lastPing = DateTime.Now;
                            _nextPing = DateTime.Now.AddMilliseconds(_delayMS / _HBCount);
                        }

                        Sleep(1000);

                    }
                }
                catch (Exception)
                {
                    Thread.Sleep(2000);
                    Cleanup();
                    InitPortComm();
                }
              

            } while (_isRunning);

            try
            {

                if (_comm != null && _comm.IsOpen)
                {
                    Debug.WriteLine("Sending KILL");
                    _comm.WriteLine("KILL");
                }
            }
            catch 
            {
                
            }
        }

        private void ProcessCommands()
        {
            string[] commands = _commandBuffer.ToString().Split('\n');
            foreach (string command in commands)
            {
                
                if (command.StartsWith("$DELAY$"))
                {
                    _delayString = command.Substring(7);
                    int.TryParse(_delayString, out _delayMS);
                    if (_delayMS > 0)
                    {
                        _nextPing = _lastPing.AddMilliseconds(_delayMS/_HBCount);
                    }
                    //Debug.WriteLine(String.Format("New Delay: {0}/{1}", _delayString, _HBCount));
                }
            }

            //check if there's any incomplete commands
            int lastIdx = _commandBuffer.ToString().LastIndexOf('\n');
            if (lastIdx != _commandBuffer.Length - 1)
            {
                _commandBuffer = new StringBuilder(commands[commands.Length-1]);
            }
            else
            {
                _commandBuffer = new StringBuilder();
            }

        }

        private void InitPortComm()
        {
            string ComPort = ConfigurationManager.AppSettings["PORT"];

            _comm = new SerialPort(ComPort, 9600);

            try
            {
                _comm.Open();
            }
            catch
            {
                Thread.Sleep(1000);
                Cleanup();
                return;
            }
            
            _comm.ReadTimeout = TimeOutMS;
            _comm.WriteTimeout = TimeOutMS;
            _comm.NewLine = "\n";
            _comm.WriteLine("DELAY");
        }


        private void Cleanup()
        {
            //clean up
            if (_comm != null)
            {
                try
                {
                    if (_comm.IsOpen)
                        _comm.Close();

                    _comm.Dispose();
                }
                catch (Exception)
                {

                }

                _comm = null;

            }

        }


        public static void Sleep(int ms)
        {

            DateTime startTime = DateTime.Now;
            DateTime endTime = startTime.AddMilliseconds(ms);
            do
            {
                Thread.Yield();
                Thread.Sleep(1);
                if (DateTime.Now >= endTime)
                    return;

            } while (true);

        }


    }
}
