﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Linq;
using System.ServiceProcess;
using System.Text;

namespace AmpDelayService
{
    public partial class ListenerService : ServiceBase
    {
        private AmpWorker _worker;

        public ListenerService()
        {
            this.CanHandlePowerEvent = true;
            this.CanPauseAndContinue = true;
            InitializeComponent();
        }

        protected override void OnStart(string[] args)
        {
            Debug.WriteLine("OnStart");
            if (_worker == null)
                _worker = new AmpWorker();

            _worker.Start();

            base.OnStart(args);
        }

        protected override void OnStop()
        {
            Debug.WriteLine("OnStop");
            _worker.Stop();
            _worker = null;

            base.OnStop();
        }

        protected override void OnShutdown()
        {
            Debug.WriteLine("OnShutdown");
            if (_worker != null)
            {
                _worker.Stop();
            }
            _worker = null;
            base.OnShutdown();
        }

        protected override void OnContinue()
        {
            Debug.WriteLine("OnContinue");
            if (_worker == null)
            {
                _worker = new AmpWorker();
            }

            _worker.Start();

            base.OnContinue();
        }

        protected override void OnPause()
        {
            Debug.WriteLine("OnPause");
            if (_worker != null)
                _worker.Stop();

            _worker = null;

            base.OnPause();
        }

        protected override bool OnPowerEvent(PowerBroadcastStatus powerStatus)
        {
            Debug.WriteLine(String.Format("OnPowerEvent: {0}", powerStatus));
            if (powerStatus == PowerBroadcastStatus.Suspend || powerStatus == PowerBroadcastStatus.Suspend)
            {
                _worker.Stop();
                _worker = null;
            }
            else if (powerStatus == PowerBroadcastStatus.ResumeSuspend)
            {
                if (_worker == null)
                {
                    _worker = new AmpWorker();
                }

                _worker.Start();
            }

            return base.OnPowerEvent(powerStatus);

        }
    }
}
